--- src/sys/dev/usb/usb_subr.c.orig	2012-07-16 00:13:31.000000000 +0300
+++ src/sys/dev/usb/usb_subr.c	2012-09-10 21:37:38.000000000 +0300
@@ -610,10 +610,8 @@
 
 	/* Get the short descriptor. */
 	err = usbd_get_config_desc(dev, index, &cd);
-	if (err) {
-		DPRINTF(("usbd_set_config_index: get_config_desc=%d\n", err));
+	if (err)
 		return (err);
-	}
 	len = UGETW(cd.wTotalLength);
 	cdp = malloc(len, M_USB, M_NOWAIT);
 	if (cdp == NULL)
@@ -626,10 +624,9 @@
 			break;
 		usbd_delay_ms(dev, 200);
 	}
-	if (err) {
-		DPRINTF(("usbd_set_config_index: get_desc=%d\n", err));
+	if (err)
 		goto bad;
-	}
+
 	if (cdp->bDescriptorType != UDESC_CONFIG) {
 		DPRINTFN(-1,("usbd_set_config_index: bad desc %d\n",
 			     cdp->bDescriptorType));
@@ -1049,34 +1046,6 @@
 }
 
 /*
- * Get the first 8 bytes of the device descriptor.
- * Do as Windows does: try to read 64 bytes -- there are devices which
- * recognize the initial descriptor fetch (before the control endpoint's
- * MaxPacketSize is known by the host) by exactly this length.
- */
-static usbd_status
-usbd_get_initial_ddesc(usbd_device_handle dev, usb_device_descriptor_t *desc)
-{
-	usb_device_request_t req;
-	char buf[64];
-	int res, actlen;
-
-	req.bmRequestType = UT_READ_DEVICE;
-	req.bRequest = UR_GET_DESCRIPTOR;
-	USETW2(req.wValue, UDESC_DEVICE, 0);
-	USETW(req.wIndex, 0);
-	USETW(req.wLength, 64);
-	res = usbd_do_request_flags(dev, &req, buf, USBD_SHORT_XFER_OK,
-		&actlen, USBD_DEFAULT_TIMEOUT);
-	if (res)
-		return res;
-	if (actlen < 8)
-		return USBD_SHORT_XFER;
-	memcpy(desc, buf, 8);
-	return USBD_NORMAL_COMPLETION;
-}
-
-/*
  * Called when a new device has been put in the powered state,
  * but not yet in the addressed state.
  * Get initial descriptor, set the address, get full descriptor,
@@ -1118,12 +1087,10 @@
 	dev->def_ep_desc.bDescriptorType = UDESC_ENDPOINT;
 	dev->def_ep_desc.bEndpointAddress = USB_CONTROL_ENDPOINT;
 	dev->def_ep_desc.bmAttributes = UE_CONTROL;
-	/*
-	 * temporary, will be fixed after first descriptor fetch
-	 * (which uses 64 bytes so it shouldn't be less),
-	 * highspeed devices must support 64 byte packets anyway
-	 */
-	USETW(dev->def_ep_desc.wMaxPacketSize, 64);
+	if (speed == USB_SPEED_HIGH)
+		USETW(dev->def_ep_desc.wMaxPacketSize, 64);
+	else
+		USETW(dev->def_ep_desc.wMaxPacketSize, USB_MAX_IPACKET);
 	dev->def_ep_desc.bInterval = 0;
 
 	/* doesn't matter, just don't let it uninitialized */
@@ -1172,7 +1139,11 @@
 	/* Try a few times in case the device is slow (i.e. outside specs.) */
 	for (i = 0; i < 10; i++) {
 		/* Get the first 8 bytes of the device descriptor. */
-		err = usbd_get_initial_ddesc(dev, dd);
+		err = usbd_get_desc(dev, UDESC_DEVICE, 0, 
+			(speed == USB_SPEED_HIGH) ? USB_DEVICE_DESCRIPTOR_SIZE
+						  : USB_MAX_IPACKET,
+			 dd);
+
 		if (!err)
 			break;
 		usbd_delay_ms(dev, 200);
